"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NatsJetStreamClientProxy = void 0;
const common_1 = require("@nestjs/common");
const microservices_1 = require("@nestjs/microservices");
const nats_1 = require("nats");
const constants_1 = require("./constants");
const nats_jetstream_record_builder_1 = require("./utils/nats-jetstream-record.builder");
const nats_request_record_builder_1 = require("./utils/nats-request-record-builder");
let NatsJetStreamClientProxy = class NatsJetStreamClientProxy extends microservices_1.ClientProxy {
    constructor(options) {
        super();
        this.options = options;
        this.codec = (0, nats_1.JSONCodec)();
    }
    async connect() {
        if (!this.nc) {
            this.nc = await (0, nats_1.connect)(this.options.connectionOptions);
            if (this.options.connectionOptions.connectedHook) {
                this.options.connectionOptions.connectedHook(this.nc);
            }
        }
        return this.nc;
    }
    async close() {
        await this.nc.drain();
        this.nc = undefined;
    }
    publish(packet, callback) {
        const subject = this.normalizePattern(packet.pattern);
        if (packet.data instanceof nats_request_record_builder_1.NatsRequestRecord) {
            const options = packet.data.options;
            const payload = this.codec.encode(packet.data.payload);
            this.nc
                .request(subject, payload, options)
                .then((msg) => this.codec.decode(msg.data))
                .then((packet) => callback(packet))
                .catch((err) => {
                callback({ err });
            });
            return () => null;
        }
        else {
            const payload = this.codec.encode(packet.data);
            this.nc
                .request(subject, payload)
                .then((msg) => this.codec.decode(msg.data))
                .then((packet) => callback(packet))
                .catch((err) => {
                callback({ err });
            });
            return () => null;
        }
    }
    async dispatchEvent(packet) {
        const subject = this.normalizePattern(packet.pattern);
        const jetStreamOpts = this.options.jetStreamOption;
        const js = this.nc.jetstream(jetStreamOpts);
        if (packet.data instanceof nats_jetstream_record_builder_1.NatsJetStreamRecord) {
            const payload = this.codec.encode(packet.data.payload);
            const options = packet.data.options;
            return js.publish(subject, payload, options);
        }
        else {
            const payload = this.codec.encode(packet.data);
            return js.publish(subject, payload);
        }
    }
};
NatsJetStreamClientProxy = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)(constants_1.NATS_JETSTREAM_OPTIONS)),
    __metadata("design:paramtypes", [Object])
], NatsJetStreamClientProxy);
exports.NatsJetStreamClientProxy = NatsJetStreamClientProxy;
//# sourceMappingURL=client.js.map